/*
** Copyright (c) 1996, 2002
**      Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that: (1) Redistributions of
** source code must retain the above copyright notice, this list of
** conditions and the following disclaimer, (2) Redistributions in
** binary form must reproduce the above copyright notice, this list
** of conditions and the following disclaimer in the documentation
** and/or other materials provided with the distribution, (3) All
** advertising materials mentioning features or use of this software
** must display the following acknowledgment: ``This product includes
** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
** The name of the author may not be used to endorse or promote products
** derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef lint
static char rcsid[] = "$Id: cmdline.c,v 1.7 2002/05/01 02:55:23 forys Exp $";
#endif

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <pwd.h>

#include <X11/IntrinsicP.h>

#include "NetintfP.h"

extern char *ProgName;

static NetintfState getargs _P_((int, char *[], NetintfWidget));

/*
 * CmdLine(argc, argv)
 *
 * If "-c" flag is present in argv, execute command to change interface
 * state, wait for state change to complete, and exit.
 * Otherwise, just return (changing nothing).
 */
void
CmdLine(argc, argv)
	int argc;
	char *argv[];
{
	char *ifmsg = "%s%s: %s: interface is %s.\n";
	static char *statestr[] = {
		"down", "init", "coming up", "running", "up",
		"coming down", "in error state"
	};
	struct _NetintfRec fakew;
	NetintfWidget w = &fakew;
	NetintfState oldstate, newstate;
	int i;

	/*
	 * Check for "-c" arg.  If not present, return immediately.
	 */
	for (i = 1; i < argc; i++)
		if (strncmp("-c", argv[i], 4) == 0)
			break;
	if (i == argc)
		return;

	/*
	 * Parse command line arguments then complete skeletal init of `w'
	 * so that we may call any of the "ifops" functions.
	 */
	newstate = getargs(argc, argv, w);
	w->netintf.sockfd = -1;
	SETSTATE(w, NetIfState(w));

	if (w->netintf.dbglvl > 0)
		fprintf(stderr, "%s: DEBUG: intf <%s> conf <%s>\n",
		        ProgName, w->netintf.ifname, w->netintf.confdir);

	/*
	 * If requested or current state is IFST_ERR, report current
	 * status and exit.
	 */
	if (newstate == IFST_ERR) {		/* report status */
		printf(ifmsg, "", ProgName, w->netintf.ifname,
		       statestr[GETSTATE(w)]);
		exit(0);
	} else if (GETSTATE(w) == IFST_ERR) {	/* interface error */
		fprintf(stderr, ifmsg, "", ProgName, w->netintf.ifname,
		        statestr[GETSTATE(w)]);
		exit(2);
	}

	/*
	 * If interface already in requested state (or, if IFST_UP was
	 * requested and the interface is already coming up), report
	 * current state and exit.
	 */
	if (GETSTATE(w) == newstate ||
	    (newstate == IFST_UP &&
	     (GETSTATE(w) == IFST_INIT || GETSTATE(w) == IFST_RUN))) {
		fprintf(stderr, "%s: %s: already %s.\n",
		        ProgName, w->netintf.ifname, statestr[GETSTATE(w)]);
		exit(1);
	}

	/*
	 * Toggle interface state.
	 */

	printf("%s: %s: bringing %s...",
		ProgName, w->netintf.ifname, statestr[newstate]);
	fflush(stdout);

	switch(newstate) {
	    case IFST_DOWN:
		NetIfDn(w);
		if (GETSTATE(w) != IFST_ERR)
			SETSTATE(w,IFST_TODOWN);
		break;
	    case IFST_UP:
		NetIfUp(w);
		if (GETSTATE(w) != IFST_ERR)
			SETSTATE(w,IFST_TOINIT);
		break;
	    default:
		break;
	}

	/*
	 * Continually report status until interface state changes,
	 * or interface reverts back to original state (failure).
	 */
	oldstate = IFST_ERR;
	while (1) {
		if (GETSTATE(w) == IFST_ERR ||
		    (newstate == IFST_UP && GETSTATE(w) == IFST_DOWN) ||
		    (newstate == IFST_DOWN && GETSTATE(w) == IFST_UP)) {
			fprintf(stderr, ifmsg, "\n", ProgName,
			        w->netintf.ifname, statestr[GETSTATE(w)]);

			exit(2);
		}

		if (GETSTATE(w) != oldstate) {
			printf(" [%s]", statestr[GETSTATE(w)]);
			fflush(stdout);
			oldstate = GETSTATE(w);
		}

		if (GETSTATE(w) == newstate) {
			break;
		} else if (GETSTATE(w) == IFST_ERR ||
		           (newstate == IFST_UP && GETSTATE(w) == IFST_DOWN) ||
		           (newstate == IFST_DOWN && GETSTATE(w) == IFST_UP)) {
			break;
		}
		sleep(IfChkinc);

		SETSTATE(w, NetIfState(w));
	}

	/*
	 * Execute commands after interface changes state.
	 */
	switch(newstate) {
	    case IFST_DOWN:
		NetIfExecDn(w);
		break;
	    case IFST_UP:
		NetIfExecUp(w);
		break;
	    default:
		break;
	}

	printf(ifmsg, "\n", ProgName, w->netintf.ifname, statestr[GETSTATE(w)]);

	exit(0);
}

/*
 * NetintfState
 * getargs(argc, argv, w);
 *
 * Parse arguments looking for those that pertain to command line mode.
 * Return requested interface state (IFST_UP, IFST_DOWN, or IFST_ERR for
 * "stat" option).  In `w', set pointer to interface name and config dir,
 * as well as debug level.
 */
static NetintfState
getargs(argc, argv, w)
	int argc;
	char *argv[];
	NetintfWidget w;
{
	NetintfState newstate;
	char *ifname, *confdir, *cp;
	int dbglvl;

	newstate = IFST_INIT;		/* any impossible state will do */
	ifname = confdir = NULL;
	dbglvl = -1;

	/*
	 * Process args that concern us; ignore everything except:
	 *	-c <up|down|stat>
	 * 	-ifname <interface>
	 * 	-confdir <directory>
	 * 	-dbglvl <level>
	 */
	for (++argv; (argc -= 2) > 0; ++argv) {
		cp = *argv++;
		if (strcmp("-c", cp) == 0) {
			if (newstate == IFST_INIT) {
				if (strcmp(*argv, "up") == 0)
					newstate = IFST_UP;
				else if (strcmp(*argv, "down") == 0)
					newstate = IFST_DOWN;
				else if (strcmp(*argv, "stat") == 0)
					newstate = IFST_ERR;
				else
					break;
			} else
				break;
		} else if (strcmp("-ifname", cp) == 0) {
			if (ifname == NULL)
				ifname = *argv;
			else
				break;
		} else if (strcmp("-confdir", cp) == 0) {
			if (confdir == NULL)
				confdir = *argv;
			else
				break;
		} else if (strcmp("-dbglvl", cp) == 0) {
			if (dbglvl == -1 && isdigit(**argv))
				dbglvl = atoi(*argv);
			else
				break;
		}
	}

	/*
	 * If argc is zero (i.e. we had an argument without a value),
	 * or positive (i.e. an error occured), report usage and exit.
	 */
	if (argc != -1) {
		fprintf(stderr, "Usage: %s -c (up|down|stat) [-ifname <if>] [-confdir <d>] [-dbglvl <l>]\n",
		        ProgName);
		exit(1);
	}

#define	MEMALL(dst, src) {						\
	if (((dst) = malloc(strlen((src)) + 2)) == NULL) {		\
		fprintf(stderr, "%s: out of memory\n", ProgName);	\
		exit(2);						\
	}								\
	(void) strcpy((dst), (src));					\
}
	/*
	 * Initialize default interface name (if not yet specified).
	 * Otherwise, just ensure that interface name has a trailing
	 * unit number (append "0" if the user did not provide one).
	 */
	if (ifname == NULL)
		MEMALL(w->netintf.ifname, IfDefault)
	else {
		for (cp = ifname; !isdigit(*cp) && *cp != '\0'; cp++)
			;
		MEMALL(w->netintf.ifname, ifname)
		if (*cp == '\0')
			(void) strcat(w->netintf.ifname, "0");
	}

	/*
	 * If no configuration directory was specified, use home dir.
	 * If home dir can not be determined, use _PATH_XNETINTF.
	 */
	if (confdir == NULL) {
		struct passwd *pw;

		if ((pw = getpwuid(getuid())) != NULL)
			MEMALL(w->netintf.confdir, pw->pw_dir)
		else
			MEMALL(w->netintf.confdir, _PATH_XNETINTF)
	} else
		MEMALL(w->netintf.confdir, confdir)

	/*
	 * Set debug level.
	 */
	w->netintf.dbglvl = (dbglvl == -1)? 0: dbglvl;
#undef	MEMALL

	return newstate;
}
